@@ -17,8 +17,6 @@
 
 
 int afs_osicred_initialized = 0;
-afs_ucred_t afs_osi_cred;
-afs_lock_t afs_xosi;		/* lock is for tvattr */
 extern struct osi_dev cacheDev;
 extern struct mount *afs_cacheVfsp;
 
@@ -34,13 +32,9 @@ osi_UFSOpen(afs_dcache_id_t *ainode)
     AFS_STATCNT(osi_UFSOpen);
     if (cacheDiskType != AFS_FCACHE_TYPE_UFS)
 	osi_Panic("UFSOpen called for non-UFS cache\n");
-    afile = (struct osi_file *)osi_AllocSmallSpace(sizeof(struct osi_file));
+    afile = osi_AllocSmallSpace(sizeof(struct osi_file));
     AFS_GUNLOCK();
-#if defined(AFS_FBSD50_ENV)
     code = VFS_VGET(afs_cacheVfsp, (ino_t) ainode->ufs, LK_EXCLUSIVE, &vp);
-#else
-    code = VFS_VGET(afs_cacheVfsp, (ino_t) ainode->ufs, &vp);
-#endif
     AFS_GLOCK();
     if (code == 0 && vp->v_type == VNON)
 	code = ENOENT;
@@ -48,13 +42,7 @@ osi_UFSOpen(afs_dcache_id_t *ainode)
 	osi_FreeSmallSpace(afile);
 	osi_Panic("UFSOpen: igetinode failed");
     }
-#if defined(AFS_FBSD80_ENV)
     VOP_UNLOCK(vp, 0);
-#elif defined(AFS_FBSD50_ENV)
-    VOP_UNLOCK(vp, 0, curthread);
-#else
-    VOP_UNLOCK(vp, 0, curproc);
-#endif
     afile->vnode = vp;
     afile->size = VTOI(vp)->i_size;
     afile->offset = 0;
@@ -63,36 +51,26 @@ osi_UFSOpen(afs_dcache_id_t *ainode)
 }
 
 int
-afs_osi_Stat(register struct osi_file *afile, register struct osi_stat *astat)
+afs_osi_Stat(struct osi_file *afile, struct osi_stat *astat)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct vattr tvattr;
     AFS_STATCNT(osi_Stat);
-    ObtainWriteLock(&afs_xosi, 320);
     AFS_GUNLOCK();
-#if defined(AFS_FBSD80_ENV)
     vn_lock(afile->vnode, LK_EXCLUSIVE | LK_RETRY);
     code = VOP_GETATTR(afile->vnode, &tvattr, afs_osi_credp);
     VOP_UNLOCK(afile->vnode, 0);
-#elif defined(AFS_FBSD50_ENV)
-    vn_lock(afile->vnode, LK_EXCLUSIVE | LK_RETRY, curthread);
-    code = VOP_GETATTR(afile->vnode, &tvattr, afs_osi_credp, curthread);
-    VOP_UNLOCK(afile->vnode, LK_EXCLUSIVE, curthread);
-#else
-    code = VOP_GETATTR(afile->vnode, &tvattr, afs_osi_credp, curproc);
-#endif
     AFS_GLOCK();
     if (code == 0) {
 	astat->size = tvattr.va_size;
 	astat->mtime = tvattr.va_mtime.tv_sec;
 	astat->atime = tvattr.va_atime.tv_sec;
     }
-    ReleaseWriteLock(&afs_xosi);
     return code;
 }
 
 int
-osi_UFSClose(register struct osi_file *afile)
+osi_UFSClose(struct osi_file *afile)
 {
     AFS_STATCNT(osi_Close);
     if (afile->vnode) {
@@ -104,14 +82,13 @@ osi_UFSClose(register struct osi_file *afile)
 }
 
 int
-osi_UFSTruncate(register struct osi_file *afile, afs_int32 asize)
+osi_UFSTruncate(struct osi_file *afile, afs_int32 asize)
 {
     struct vattr tvattr;
     struct vnode *vp;
-    register afs_int32 code, glocked;
+    afs_int32 code, glocked;
     AFS_STATCNT(osi_Truncate);
 
-    ObtainWriteLock(&afs_xosi, 321);
     vp = afile->vnode;
     /*
      * This routine only shrinks files, and most systems
@@ -121,40 +98,19 @@ osi_UFSTruncate(register struct osi_file *afile, afs_int32 asize)
     glocked = ISAFS_GLOCK();
     if (glocked)
       AFS_GUNLOCK();
-#if defined(AFS_FBSD80_ENV)
     vn_lock(vp, LK_EXCLUSIVE | LK_RETRY);
     code = VOP_GETATTR(afile->vnode, &tvattr, afs_osi_credp);
-#elif defined(AFS_FBSD50_ENV)
-    vn_lock(vp, LK_EXCLUSIVE | LK_RETRY, curthread);
-    code = VOP_GETATTR(afile->vnode, &tvattr, afs_osi_credp, curthread);
-#else
-    vn_lock(vp, LK_EXCLUSIVE | LK_RETRY, curproc);
-    code = VOP_GETATTR(afile->vnode, &tvattr, afs_osi_credp, curproc);
-#endif
     if (code != 0 || tvattr.va_size <= asize)
 	goto out;
 
     VATTR_NULL(&tvattr);
     tvattr.va_size = asize;
-#if defined(AFS_FBSD80_ENV)
     code = VOP_SETATTR(vp, &tvattr, afs_osi_credp);
-#elif defined(AFS_FBSD50_ENV)
-    code = VOP_SETATTR(vp, &tvattr, afs_osi_credp, curthread);
-#else
-    code = VOP_SETATTR(vp, &tvattr, afs_osi_credp, curproc);
-#endif
 
 out:
-#if defined(AFS_FBSD80_ENV)
     VOP_UNLOCK(vp, 0);
-#elif defined(AFS_FBSD50_ENV)
-    VOP_UNLOCK(vp, LK_EXCLUSIVE, curthread);
-#else
-    VOP_UNLOCK(vp, LK_EXCLUSIVE, curproc);
-#endif
     if (glocked)
       AFS_GLOCK();
-    ReleaseWriteLock(&afs_xosi);
     return code;
 }
 
@@ -168,11 +124,15 @@ osi_DisableAtimes(struct vnode *avp)
 
 /* Generic read interface */
 int
-afs_osi_Read(register struct osi_file *afile, int offset, void *aptr,
+afs_osi_Read(struct osi_file *afile, int offset, void *aptr,
 	     afs_int32 asize)
 {
-    unsigned int resid;
-    register afs_int32 code;
+#if (__FreeBSD_version >= 900505 && __FreeBSD_Version < 1000000) ||__FreeBSD_version >= 1000009
+    ssize_t resid;
+#else
+    int resid;
+#endif
+    afs_int32 code;
     AFS_STATCNT(osi_Read);
 
     /**
@@ -180,10 +140,10 @@ afs_osi_Read(register struct osi_file *afile, int offset, void *aptr,
       * down. No point in crashing when we are already shutting down
       */
     if (!afile) {
-	if (!afs_shuttingdown)
+	if (afs_shuttingdown == AFS_RUNNING)
 	    osi_Panic("osi_Read called with null param");
 	else
-	    return EIO;
+	    return -EIO;
     }
 
     if (offset != -1)
@@ -198,20 +158,26 @@ afs_osi_Read(register struct osi_file *afile, int offset, void *aptr,
 	afile->offset += code;
 	osi_DisableAtimes(afile->vnode);
     } else {
-	afs_Trace2(afs_iclSetp, CM_TRACE_READFAILED, ICL_TYPE_INT32, resid,
+	afs_Trace2(afs_iclSetp, CM_TRACE_READFAILED, ICL_TYPE_INT32, (int)resid,
 		   ICL_TYPE_INT32, code);
-	code = -1;
+	if (code > 0) {
+	    code = -code;
+	}
     }
     return code;
 }
 
 /* Generic write interface */
 int
-afs_osi_Write(register struct osi_file *afile, afs_int32 offset, void *aptr,
+afs_osi_Write(struct osi_file *afile, afs_int32 offset, void *aptr,
 	      afs_int32 asize)
 {
-    unsigned int resid;
-    register afs_int32 code;
+#if (__FreeBSD_version >= 900505 && __FreeBSD_Version < 1000000) ||__FreeBSD_version >= 1000009
+    ssize_t resid;
+#else
+    int resid;
+#endif
+    afs_int32 code;
     AFS_STATCNT(osi_Write);
     if (!afile)
 	osi_Panic("afs_osi_Write called with null param");
@@ -229,7 +195,9 @@ afs_osi_Write(register struct osi_file *afile, afs_int32 offset, void *aptr,
 	code = asize - resid;
 	afile->offset += code;
     } else {
-	code = -1;
+	if (code > 0) {
+	    code = -code;
+	}
     }
     if (afile->proc) {
 	(*afile->proc) (afile, code);
@@ -242,7 +210,7 @@ afs_osi_Write(register struct osi_file *afile, afs_int32 offset, void *aptr,
     This routine written from the RT NFS port strategy routine.
     It has been generalized a bit, but should still be pretty clear. */
 int
-afs_osi_MapStrategy(int (*aproc) (), register struct buf *bp)
+afs_osi_MapStrategy(int (*aproc) (), struct buf *bp)
 {
     afs_int32 returnCode;
 
