@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -52,21 +52,17 @@ static struct ltable {
     "afs_xcbhash", (char *)&afs_xcbhash}, {
     "afs_xaxs", (char *)&afs_xaxs}, {
     "afs_xinterface", (char *)&afs_xinterface},
-#ifndef UKERNEL
-    {
-    "afs_xosi", (char *)&afs_xosi},
-#endif
     {
       "afs_xsrvAddr", (char *)&afs_xsrvAddr},
     {
       "afs_xvreclaim", (char *)&afs_xvreclaim},
     { "afsdb_client_lock", (char *)&afsdb_client_lock},
     { "afsdb_req_lock", (char *)&afsdb_req_lock},
-#ifdef AFS_DISCON_ENV
     { "afs_discon_lock", (char *)&afs_discon_lock},
     { "afs_disconDirtyLock", (char *)&afs_disconDirtyLock},
     { "afs_discon_vc_dirty", (char *)&afs_xvcdirty},
-#endif
+    { "afs_dynrootDirLock", (char *)&afs_dynrootDirLock},
+    { "afs_dynSymlinkLock", (char *)&afs_dynSymlinkLock},
 };
 unsigned long lastCallBack_vnode;
 unsigned int lastCallBack_dv;
@@ -103,8 +99,8 @@ SRXAFSCB_GetCE(struct rx_call *a_call, afs_int32 a_index,
 	       struct AFSDBCacheEntry *a_result)
 {
 
-    register int i;		/*Loop variable */
-    register struct vcache *tvc;	/*Ptr to current cache entry */
+    int i;		/*Loop variable */
+    struct vcache *tvc;	/*Ptr to current cache entry */
     int code;			/*Return code */
     XSTATS_DECLS;
 
@@ -189,8 +185,8 @@ int
 SRXAFSCB_GetCE64(struct rx_call *a_call, afs_int32 a_index,
 		 struct AFSDBCacheEntry64 *a_result)
 {
-    register int i;		/*Loop variable */
-    register struct vcache *tvc;	/*Ptr to current cache entry */
+    int i;		/*Loop variable */
+    struct vcache *tvc;	/*Ptr to current cache entry */
     int code;			/*Return code */
     XSTATS_DECLS;
 
@@ -236,12 +232,7 @@ SRXAFSCB_GetCE64(struct rx_call *a_call, afs_int32 a_index,
     a_result->lock.pid_writer = 0;
     a_result->lock.src_indicator = 0;
 #endif /* INSTRUMENT_LOCKS */
-#if !defined(AFS_64BIT_ENV) 
-    a_result->Length.high = 0;
-    a_result->Length.low = tvc->f.m.Length;
-#else
     a_result->Length = tvc->f.m.Length;
-#endif
     a_result->DataVersion = hgetlo(tvc->f.m.DataVersion);
     a_result->callback = afs_data_pointer_to_int32(tvc->callback);	/* XXXX Now a pointer; change it XXXX */
     a_result->cbExpires = tvc->cbExpires;
@@ -311,6 +302,7 @@ SRXAFSCB_GetLock(struct rx_call *a_call, afs_int32 a_index,
     XSTATS_START_CMTIME(AFS_STATS_CM_RPCIDX_GETLOCK);
 
     AFS_STATCNT(SRXAFSCB_GetLock);
+    memset(a_result, 0, sizeof(*a_result));
     nentries = sizeof(ltable) / sizeof(struct ltable);
     if (a_index < 0 || a_index >= nentries+afs_cellindex) {
 	/*
@@ -384,7 +376,7 @@ SRXAFSCB_GetLock(struct rx_call *a_call, afs_int32 a_index,
  *
  * Description:
  *	Clear out callback information for the specified file, or
- *	even a whole volume.  Used to worry about callback was from 
+ *	even a whole volume.  Used to worry about callback was from
  *      within the particular cell or not.  Now we don't bother with
  *      that anymore; it's not worth the time.
  *
@@ -406,11 +398,11 @@ Appears to need to be called with GLOCK held, as the icl_Event4 stuff asserts ot
  *------------------------------------------------------------------------*/
 
 static int
-ClearCallBack(register struct rx_connection *a_conn,
-	      register struct AFSFid *a_fid)
+ClearCallBack(struct rx_connection *a_conn,
+	      struct AFSFid *a_fid)
 {
-    register struct vcache *tvc;
-    register int i;
+    struct vcache *tvc;
+    int i;
     struct VenusFid localFid;
     struct volume *tv;
 #ifdef AFS_DARWIN80_ENV
@@ -444,7 +436,7 @@ loop1:
 		i = VCHashV(&localFid);
 		for (tq = afs_vhashTV[i].prev; tq != &afs_vhashTV[i]; tq = uq) {
 		    uq = QPrev(tq);
-		    tvc = QTOVH(tq);      
+		    tvc = QTOVH(tq);
 		    if (tvc->f.fid.Fid.Volume == a_fid->Volume) {
 			tvc->callback = NULL;
 			if (!localFid.Cell)
@@ -456,15 +448,13 @@ loop1:
 			    goto loop1;
 			}
 #if     defined(AFS_SGI_ENV) || defined(AFS_SUN5_ENV)  || defined(AFS_HPUX_ENV) || defined(AFS_LINUX20_ENV)
-			VN_HOLD(AFSTOV(tvc));
+			AFS_FAST_HOLD(tvc);
 #else
 #ifdef AFS_DARWIN80_ENV
 			if (tvc->f.states & CDeadVnode) {
-			    if (!(tvc->f.states & CBulkFetching)) {
-				ReleaseReadLock(&afs_xvcache);
-				afs_osi_Sleep(&tvc->f.states);
-				goto loop1;
-			    }
+			    ReleaseReadLock(&afs_xvcache);
+			    afs_osi_Sleep(&tvc->f.states);
+			    goto loop1;
 			}
 			vp = AFSTOV(tvc);
 			if (vnode_get(vp))
@@ -475,27 +465,17 @@ loop1:
 			    AFS_GLOCK();
 			    continue;
 			}
-			if (tvc->f.states & (CBulkFetching|CDeadVnode)) {
-			    AFS_GUNLOCK();
-			    vnode_recycle(AFSTOV(tvc));
-			    AFS_GLOCK();
-			}
 #else
 			AFS_FAST_HOLD(tvc);
 #endif
 #endif
 			ReleaseReadLock(&afs_xvcache);
-			ObtainWriteLock(&afs_xcbhash, 449);
-			afs_DequeueCallback(tvc);
-			tvc->f.states &= ~(CStatd | CUnique | CBulkFetching);
+			afs_StaleVCacheFlags(tvc, 0, CUnique | CBulkFetching);
 			afs_allCBs++;
 			if (tvc->f.fid.Fid.Vnode & 1)
 			    afs_oddCBs++;
 			else
 			    afs_evenCBs++;
-			ReleaseWriteLock(&afs_xcbhash);
-			if ((tvc->f.fid.Fid.Vnode & 1 || (vType(tvc) == VDIR)))
-			    osi_dnlc_purgedp(tvc);
 			afs_Trace3(afs_iclSetp, CM_TRACE_CALLBACK,
 				   ICL_TYPE_POINTER, tvc, ICL_TYPE_INT32,
 				   tvc->f.states, ICL_TYPE_INT32,
@@ -507,10 +487,10 @@ loop1:
 			uq = QPrev(tq);
 			AFS_FAST_RELE(tvc);
 		    } else if ((tvc->f.states & CMValid)
-			       && (tvc->mvid->Fid.Volume == a_fid->Volume)) {
+			       && (tvc->mvid.target_root->Fid.Volume == a_fid->Volume)) {
 			tvc->f.states &= ~CMValid;
 			if (!localFid.Cell)
-			    localFid.Cell = tvc->mvid->Cell;
+			    localFid.Cell = tvc->mvid.target_root->Cell;
 		    }
 		}
 		ReleaseReadLock(&afs_xvcache);
@@ -551,15 +531,13 @@ loop2:
 			goto loop2;
 		    }
 #if     defined(AFS_SGI_ENV) || defined(AFS_SUN5_ENV)  || defined(AFS_HPUX_ENV) || defined(AFS_LINUX20_ENV)
-		    VN_HOLD(AFSTOV(tvc));
+		    AFS_FAST_HOLD(tvc);
 #else
 #ifdef AFS_DARWIN80_ENV
 		    if (tvc->f.states & CDeadVnode) {
-			if (!(tvc->f.states & CBulkFetching)) {
-			    ReleaseReadLock(&afs_xvcache);
-			    afs_osi_Sleep(&tvc->f.states);
-			    goto loop2;
-			}
+			ReleaseReadLock(&afs_xvcache);
+			afs_osi_Sleep(&tvc->f.states);
+			goto loop2;
 		    }
 		    vp = AFSTOV(tvc);
 		    if (vnode_get(vp))
@@ -570,22 +548,12 @@ loop2:
 			AFS_GLOCK();
 			continue;
 		    }
-		    if (tvc->f.states & (CBulkFetching|CDeadVnode)) {
-			AFS_GUNLOCK();
-			vnode_recycle(AFSTOV(tvc));
-			AFS_GLOCK();
-		    }
 #else
 		    AFS_FAST_HOLD(tvc);
 #endif
 #endif
 		    ReleaseReadLock(&afs_xvcache);
-		    ObtainWriteLock(&afs_xcbhash, 450);
-		    afs_DequeueCallback(tvc);
-		    tvc->f.states &= ~(CStatd | CUnique | CBulkFetching);
-		    ReleaseWriteLock(&afs_xcbhash);
-		    if ((tvc->f.fid.Fid.Vnode & 1 || (vType(tvc) == VDIR)))
-			osi_dnlc_purgedp(tvc);
+		    afs_StaleVCacheFlags(tvc, 0, CUnique | CBulkFetching);
 		    afs_Trace3(afs_iclSetp, CM_TRACE_CALLBACK,
 			       ICL_TYPE_POINTER, tvc, ICL_TYPE_INT32,
 			       tvc->f.states, ICL_TYPE_LONG, 0);
@@ -639,12 +607,12 @@ loop2:
  *------------------------------------------------------------------------*/
 
 int
-SRXAFSCB_CallBack(struct rx_call *a_call, register struct AFSCBFids *a_fids,
+SRXAFSCB_CallBack(struct rx_call *a_call, struct AFSCBFids *a_fids,
 		  struct AFSCBs *a_callbacks)
 {
-    register int i;		/*Loop variable */
+    int i;		/*Loop variable */
     struct AFSFid *tfid;	/*Ptr to current fid */
-    register struct rx_connection *tconn;	/*Call's connection */
+    struct rx_connection *tconn;	/*Call's connection */
     int code = 0;
     XSTATS_DECLS;
 
@@ -736,10 +704,10 @@ SRXAFSCB_Probe(struct rx_call *a_call)
 int
 SRXAFSCB_InitCallBackState(struct rx_call *a_call)
 {
-    register int i;
-    register struct vcache *tvc;
-    register struct rx_connection *tconn;
-    register struct rx_peer *peer;
+    int i;
+    struct vcache *tvc;
+    struct rx_connection *tconn;
+    struct rx_peer *peer;
     struct server *ts;
     int code = 0;
     XSTATS_DECLS;
@@ -764,21 +732,22 @@ SRXAFSCB_InitCallBackState(struct rx_call *a_call)
 	    for (i = 0; i < VCSIZE; i++)
 		for (tvc = afs_vhashT[i]; tvc; tvc = tvc->hnext) {
 		    if (tvc->callback == ts) {
-			ObtainWriteLock(&afs_xcbhash, 451);
-			afs_DequeueCallback(tvc);
-			tvc->callback = NULL;
-			tvc->f.states &= ~(CStatd | CUnique | CBulkFetching);
-			ReleaseWriteLock(&afs_xcbhash);
+			afs_StaleVCacheFlags(tvc, AFS_STALEVC_NODNLC |
+						  AFS_STALEVC_CLEARCB,
+					     CUnique | CBulkFetching);
 		    }
 		}
+
+	    /* capabilities need be requested again */
+	    ts->flags &= ~SCAPS_KNOWN;
 	}
 
 
 
 	/* find any volumes residing on this server and flush their state */
 	{
-	    register struct volume *tv;
-	    register int j;
+	    struct volume *tv;
+	    int j;
 
 	    for (i = 0; i < NVOLS; i++)
 		for (tv = afs_volumes[i]; tv; tv = tv->next) {
@@ -872,7 +841,7 @@ SRXAFSCB_GetXStats(struct rx_call *a_call, afs_int32 a_clientVersionNum,
 		   afs_int32 a_collectionNumber, afs_int32 * a_srvVersionNumP,
 		   afs_int32 * a_timeP, AFSCB_CollData * a_dataP)
 {
-    register int code;		/*Return value */
+    int code;		/*Return value */
     afs_int32 *dataBuffP;	/*Ptr to data to be returned */
     afs_int32 dataBytes;	/*Bytes in data buffer */
     XSTATS_DECLS;
@@ -911,6 +880,7 @@ SRXAFSCB_GetXStats(struct rx_call *a_call, afs_int32 a_clientVersionNum,
 	 */
 	dataBytes = sizeof(struct afs_CMStats);
 	dataBuffP = (afs_int32 *) afs_osi_Alloc(dataBytes);
+	osi_Assert(dataBuffP != NULL);
 	memcpy((char *)dataBuffP, (char *)&afs_cmstats, dataBytes);
 	a_dataP->AFSCB_CollData_len = dataBytes >> 2;
 	a_dataP->AFSCB_CollData_val = dataBuffP;
@@ -931,6 +901,7 @@ SRXAFSCB_GetXStats(struct rx_call *a_call, afs_int32 a_clientVersionNum,
 	afs_CountServers();
 	dataBytes = sizeof(afs_stats_cmperf);
 	dataBuffP = (afs_int32 *) afs_osi_Alloc(dataBytes);
+	osi_Assert(dataBuffP != NULL);
 	memcpy((char *)dataBuffP, (char *)&afs_stats_cmperf, dataBytes);
 	a_dataP->AFSCB_CollData_len = dataBytes >> 2;
 	a_dataP->AFSCB_CollData_val = dataBuffP;
@@ -955,6 +926,7 @@ SRXAFSCB_GetXStats(struct rx_call *a_call, afs_int32 a_clientVersionNum,
 
 	dataBytes = sizeof(afs_stats_cmfullperf);
 	dataBuffP = (afs_int32 *) afs_osi_Alloc(dataBytes);
+	osi_Assert(dataBuffP != NULL);
 	memcpy((char *)dataBuffP, (char *)(&afs_stats_cmfullperf), dataBytes);
 	a_dataP->AFSCB_CollData_len = dataBytes >> 2;
 	a_dataP->AFSCB_CollData_val = dataBuffP;
@@ -1062,7 +1034,7 @@ shutdown_CB(void)
  *      a_call : Ptr to Rx call on which this request came in.
  *
  * Returns:
- *      RXGEN_OPCODE (always). 
+ *      RXGEN_OPCODE (always).
  *
  * Environment:
  *      Nothing interesting.
@@ -1308,8 +1280,8 @@ SRXAFSCB_GetCellServDB(struct rx_call *a_call, afs_int32 a_index,
 	p_name = tcell->cellName;
 	for (j = 0; j < AFSMAXCELLHOSTS && tcell->cellHosts[j]; j++);
 	i = strlen(p_name);
-	a_hosts->serverList_val =
-	    (afs_int32 *) afs_osi_Alloc(j * sizeof(afs_int32));
+	a_hosts->serverList_val = afs_osi_Alloc(j * sizeof(afs_int32));
+	osi_Assert(a_hosts->serverList_val != NULL);
 	a_hosts->serverList_len = j;
 	for (j = 0; j < AFSMAXCELLHOSTS && tcell->cellHosts[j]; j++)
 	    a_hosts->serverList_val[j] =
@@ -1317,9 +1289,10 @@ SRXAFSCB_GetCellServDB(struct rx_call *a_call, afs_int32 a_index,
 	afs_PutCell(tcell, READ_LOCK);
     }
 
-    t_name = (char *)afs_osi_Alloc(i + 1);
+    t_name = afs_osi_Alloc(i + 1);
     if (t_name == NULL) {
-	afs_osi_Free(a_hosts->serverList_val, (j * sizeof(afs_int32)));
+	if (tcell != NULL)
+	    afs_osi_Free(a_hosts->serverList_val, (j * sizeof(afs_int32)));
 	RX_AFS_GUNLOCK();
 	return ENOMEM;
     }
@@ -1375,7 +1348,7 @@ SRXAFSCB_GetLocalCell(struct rx_call *a_call, char **a_name)
 	plen = strlen(p_name);
     else
 	plen = 0;
-    t_name = (char *)afs_osi_Alloc(plen + 1);
+    t_name = afs_osi_Alloc(plen + 1);
     if (t_name == NULL) {
 	if (tcell)
 	    afs_PutCell(tcell, READ_LOCK);
@@ -1472,7 +1445,7 @@ SRXAFSCB_GetCacheConfig(struct rx_call *a_call, afs_uint32 callerVersion,
      * Currently only support version 1
      */
     allocsize = sizeof(cm_initparams_v1);
-    t_config = (afs_uint32 *) afs_osi_Alloc(allocsize);
+    t_config = afs_osi_Alloc(allocsize);
     if (t_config == NULL) {
 	RX_AFS_GUNLOCK();
 	return ENOMEM;
@@ -1613,8 +1586,8 @@ SRXAFSCB_GetCellByNum(struct rx_call *a_call, afs_int32 a_cellnum,
 
     for (sn = 0; sn < AFSMAXCELLHOSTS && tcell->cellHosts[sn]; sn++);
     a_hosts->serverList_len = sn;
-    a_hosts->serverList_val =
-	(afs_int32 *) afs_osi_Alloc(sn * sizeof(afs_int32));
+    a_hosts->serverList_val = afs_osi_Alloc(sn * sizeof(afs_int32));
+    osi_Assert(a_hosts->serverList_val != NULL);
 
     for (i = 0; i < sn; i++)
 	a_hosts->serverList_val[i] = ntohl(tcell->cellHosts[i]->addr->sa_ip);
@@ -1642,6 +1615,7 @@ SRXAFSCB_TellMeAboutYourself(struct rx_call *a_call,
     ObtainReadLock(&afs_xinterface);
 
     /* return all network interface addresses */
+    memset(addr, 0, sizeof(*addr));
     addr->numberOfInterfaces = afs_cb_interface.numberOfInterfaces;
     addr->uuid = afs_cb_interface.uuid;
     for (i = 0; i < afs_cb_interface.numberOfInterfaces; i++) {
@@ -1655,7 +1629,8 @@ SRXAFSCB_TellMeAboutYourself(struct rx_call *a_call,
     RX_AFS_GUNLOCK();
 
     dataBytes = 1 * sizeof(afs_uint32);
-    dataBuffP = (afs_uint32 *) afs_osi_Alloc(dataBytes);
+    dataBuffP = afs_osi_Alloc(dataBytes);
+    osi_Assert(dataBuffP != NULL);
     dataBuffP[0] = CLIENT_CAPABILITY_ERRORTRANS;
     capabilities->Capabilities_len = dataBytes / sizeof(afs_uint32);
     capabilities->Capabilities_val = dataBuffP;
@@ -1664,134 +1639,12 @@ SRXAFSCB_TellMeAboutYourself(struct rx_call *a_call,
 }
 
 
-#if 0 && defined(AFS_LINUX24_ENV)
-extern struct vcache *afs_globalVp;
-
-int recurse_dcache_parent(parent, a_index, addr, inode, flags, time, fileName)
-     struct dentry * parent;
-    afs_int32 a_index;
-    afs_int32 *addr;
-    afs_int32 *inode;
-    afs_int32 *flags;
-    afs_int32 *time;
-    char ** fileName;
-{
-	struct dentry *this_parent = parent;
-	struct list_head *next;
-	int found = 0;
-	struct dentry *dentry;
-
-repeat:
-	next = this_parent->d_subdirs.next;
-resume:
-	while (next != &this_parent->d_subdirs) {
-		struct list_head *tmp = next;
-		dentry = list_entry(tmp, struct dentry, d_child);
-		if (a_index == 0)
-		  goto searchdone3;
-		a_index--;
-		next = tmp->next;
-		/*
-		 * Descend a level if the d_subdirs list is non-empty.
-		 */
-		if (!list_empty(&dentry->d_subdirs)) {
-			this_parent = dentry;
-			goto repeat;
-		}
-	}
-	/*
-	 * All done at this level ... ascend and resume the search.
-	 */
-	if (this_parent != parent) {
-		next = this_parent->d_child.next; 
-		this_parent = this_parent->d_parent;
-		goto resume;
-	}
-	goto ret;
-
- searchdone3:
-    if (d_unhashed(dentry))
-      *flags = 1;
-    else 
-      *flags = 0;
-
-    *fileName = afs_strdup(dentry->d_name.name?dentry->d_name.name:"");
-    *inode = ITOAFS(dentry->d_inode);
-    *addr = atomic_read(&(dentry)->d_count);
-    *time = dentry->d_time;
-
-    return 0;
- ret:
-    return 1;
-}
-#endif
-
-int SRXAFSCB_GetDE(struct rx_call *a_call, afs_int32 a_index, afs_int32 *addr,
-		   afs_int32 *inode, afs_int32 *flags, afs_int32 *time,
-		   char ** fileName)
+int
+SRXAFSCB_GetDE(struct rx_call *a_call, afs_int32 a_index, afs_int32 *addr,
+	       afs_int32 *inode, afs_int32 *flags, afs_int32 *time,
+	       char ** fileName)
 { /*SRXAFSCB_GetDE*/
     int code = 0;				/*Return code*/
-#if 0 && defined(AFS_LINUX24_ENV)
-    register int i;			/*Loop variable*/
-    register struct vcache *tvc = afs_globalVp;
-    struct dentry *dentry;
-    struct list_head *cur, *head = &(AFSTOI(tvc))->i_dentry;
-
-#ifdef RX_ENABLE_LOCKS
-    AFS_GLOCK();
-#endif /* RX_ENABLE_LOCKS */
-
-#if defined(AFS_LINUX24_ENV)
-    spin_lock(&dcache_lock);
-#endif
-
-    cur = head;
-    while ((cur = cur->next) != head) {
-      dentry = list_entry(cur, struct dentry, d_alias);
-      
-      dget_locked(dentry);
-      
-#if defined(AFS_LINUX24_ENV)
-      spin_unlock(&dcache_lock);
-#endif
-      if (a_index == 0)
-	goto searchdone2;
-      a_index--;
-
-      if (recurse_dcache_parent(dentry, a_index, addr, inode, flags, time, fileName) == 0) {
-	dput(dentry);
-	code = 0;
-	goto fcnDone;
-      }
-      dput(dentry);
-    }                   
- searchdone2:
-    if (cur == head) {
-	/*Past EOF*/
-	code = 1;
-	*fileName = afs_strdup("");
-	goto fcnDone;
-    }
-
-    if (d_unhashed(dentry))
-      *flags = 1;
-    else 
-      *flags = 0;
-
-    *fileName = afs_strdup(dentry->d_name.name?dentry->d_name.name:"");
-    *inode = ITOAFS(dentry->d_inode);
-    *addr = atomic_read(&(dentry)->d_count);
-    *time = dentry->d_time;
-
-    dput(dentry);
-    code = 0;
-
-fcnDone:
-
-#ifdef RX_ENABLE_LOCKS
-    AFS_GUNLOCK();
-#endif /* RX_ENABLE_LOCKS */
-#endif
     return(code);
 
 } /*SRXAFSCB_GetDE*/
